// A simple 8-bit accumulator
// reset=0 -> set acc to 0
// reset=1 -> add "in" to acc
//
// This chip has 20 pins (5 on each side) and 4 corners
// PADNC can be used to fill unused pins
//
// For Pads: DO   is the "Data Out" pin
//           DI   is the "Data In" pin
//           YPAD is the bonding-pad (IC package)
//
// Johannes Grad
// gradjoh@iit.edu
//

//
// The core 
//

module accu_core(in, acc, clk, reset);
input [7:0] in;
input clk, reset;
output [7:0] acc;
reg [7:0] acc;

always @(posedge clk)
begin
	if (reset) acc<=0; 
	else acc<=acc+in;
end

endmodule


//
// The Toplevel
// 

module accu(in, out, clk, reset);
input  [7:0] in;
output [7:0] out;
input clk, reset;

wire [7:0] in_i, out_i;
wire clk_i, reset_i;

accu_core my_core(in_i, out_i, clk_i, reset_i);

PADINC   p19(.DI(clk_i), .YPAD(clk));
PADINC   p20(.DI(reset_i), .YPAD(reset));

PADGND   p17();
PADVDD   p18();

PADOUT p01(.DO(out_i[0]), .YPAD(out[0]));
PADOUT p02(.DO(out_i[1]), .YPAD(out[1]));
PADOUT p03(.DO(out_i[2]), .YPAD(out[2]));
PADOUT p04(.DO(out_i[3]), .YPAD(out[3]));
PADOUT p05(.DO(out_i[4]), .YPAD(out[4]));
PADOUT p06(.DO(out_i[5]), .YPAD(out[5]));
PADOUT p07(.DO(out_i[6]), .YPAD(out[6]));
PADOUT p08(.DO(out_i[7]), .YPAD(out[7]));

PADINC   p09(.DI(in_i[0]), .YPAD(in[0]));
PADINC   p10(.DI(in_i[1]), .YPAD(in[1]));
PADINC   p11(.DI(in_i[2]), .YPAD(in[2]));
PADINC   p12(.DI(in_i[3]), .YPAD(in[3]));
PADINC   p13(.DI(in_i[4]), .YPAD(in[4]));
PADINC   p14(.DI(in_i[5]), .YPAD(in[5]));
PADINC   p15(.DI(in_i[6]), .YPAD(in[6]));
PADINC   p16(.DI(in_i[7]), .YPAD(in[7]));

PADFC	 c01();
PADFC	 c02();
PADFC	 c03();
PADFC	 c04();

endmodule
